///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 3: Bzier      //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2005 (C)            //
//                                           //
///////////////////////////////////////////////
#include "../data/common.h"
#include "bezier.h"
#pragma comment(lib, "opengl32.lib")



//////////////
// #DEFINES //
//////////////
#define MODULE_NAME "Bezier curves"



//////////////
// ACCURACY //
//////////////
#define ACCURACY 2000  // Lines per fragment
#define POINTS   6     // Control points



//////////
// MAIN //
//////////

int main(int argc, char *argv[])
{ 
   /////////////////
   // Init MVisio //
   /////////////////
   INIT_MVISIO(MODULE_NAME, argc, argv);   
   
   // Common interface:
   USE_FONT();  
   USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);
  
   
   ////////////
   // Curves //
   ////////////     
   
   // Interface:
   MVBUTTON *point[POINTS];
   for (int c=0; c<POINTS; c++)
   {
         point[c] = new MVBUTTON();
         point[c]->setFont(font);
         point[c]->setSizes(16, 16);
         point[c]->setStaticStatus(false);
         point[c]->setLimitStatus(true);
         point[c]->setLimits(5, 44, MODULE_SIZEX-5, MODULE_SIZEY-95);
         char _pname[20];
         sprintf(_pname, "%d", c+1);
         point[c]->setCaption(_pname);
         if (c==0 || c==POINTS-1)
               point[c]->setColor(0.0f, 0.8f, 0.0f, 0.5f);
         else
               point[c]->setColor(0.5f, 0.5f, 0.0f, 0.5f);
         point[c]->setPosition((float) ((c+1)*MODULE_SIZEX/(POINTS+1)), (float) (MODULE_SIZEY/2));
   }
   
   // Control points:
   MVVECTOR3 cPoint[POINTS]; 
   EXF_ZEROMEMORY(cPoint, sizeof(MVVECTOR3)*POINTS);   
  
  
   ////////////////
   // Module slide:
   MVTEXTURE *slideTexture = new MVTEXTURE();
   slideTexture->load("beziercurves_slide.tga");
   MVWINDOW *slideWindow = new MVWINDOW();
   slideWindow->setFont(font);
   slideWindow->setCaption("Slide");
   slideWindow->setSizes(512, 440);
   slideWindow->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   slideWindow->setTexture(slideTexture);
   slideWindow->setPosition(15, 85);
   
   MVVSCROLLBAR *slideScroll = new MVVSCROLLBAR();
   slideScroll->setMinValue(0.1f);
   slideScroll->setMaxValue(1.0f);
   slideScroll->setValue(1.0f);
   slideScroll->setPosition(495, 12);
   slideScroll->link(slideWindow);   
            
  
///////////////
// Mainloop: //
///////////////

   int mouseX = 0, mouseY = 0;
   bool mouseBL = false, mouseBR = false;
   bool done = false;
   while (!done)   
	{			
	      // Rendering:	   
	   	MVISIO::clear(true, true, false);	   		   	
			MVISIO::begin2D();			   
			   PASS_BACKGROUND;					     			  
			MVISIO::end2D();				     
			
			// Draw spline:			
			MVISIO::begin2D();		
			   
			   // Title:
			   font->putString(10, 13, MODULE_NAME);	  
			   glDisable(GL_TEXTURE_2D);
			   
			   Table_triplet t;
			   t.nb = POINTS;
			   t.table = (Triplet *) malloc(sizeof(Triplet) * t.nb);
			   for (int c=0; c<POINTS; c++)
			   {
			         MVVECTOR2 p0 = MVVECTOR2(point[c]->getPosition().x, point[c]->getPosition().y);
			         p0 += MVVECTOR2((float) point[c]->getSizeX()/2.0f, (float) point[c]->getSizeY()/2.0f);		         			         
			         t.table[c].x = p0.x; t.table[c].y = p0.y;
			   }
			   
			   // Compute line:
			   Table_triplet bezier;
			   bezier = calcul_courbe_bezier(t, ACCURACY);
			   glLineWidth(LINE_WIDTH);
			   glBegin(GL_LINE_STRIP);			   
			   for (int c=0; c<bezier.nb; c++)
			   {
			         glColor4f(1.0f, 1.0f, 0.0f, 1.0f);	
			         glVertex2f(bezier.table[c].x, bezier.table[c].y);
			   }
			   glEnd();
			   free(bezier.table);
			MVISIO::end2D();
			
			   free(t.table);
			   
			// Draw control points:
			MVISIO::begin2D();
			   PASS_QUITBUTTON;			   
			   for (int c=0; c<POINTS; c++)
			         point[c]->pass();		
			   slideWindow->setColor(1.0f, 1.0f, 1.0f, slideScroll->getValue());
			   slideWindow->pass();	  
			MVISIO::end2D();
			
			////////////////
			// Check events:			
			SDL_Event event;			
			//SDL_PumpEvents();						
			while (SDL_PollEvent(&event)) 
			{
					// Quit by click on X:
					if (event.type == SDL_QUIT) 
							done = true;		
               
               // Resize?
				   if (event.type == SDL_VIDEORESIZE)
				         MVSETTINGS::resize(0, 0, event.resize.w, event.resize.h);														
							
               // Quit by pressing ESC:
					if (event.type == SDL_KEYDOWN) 
					{
							// Quit by ESC:
							if (event.key.keysym.sym == SDLK_ESCAPE) 
							       done = true;															
			      }														
			}         
			
			// Update mouse:
			if (SDL_GetMouseState(&mouseX, &mouseY)&SDL_BUTTON(1))
					mouseBL = true;
			else 
					mouseBL = false;
			
			// Check MVisio events:
			MVID id=0;
			MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);  
			if (PRESSED_QUITBUTTON(id))
			      done = true;
			
			// Display:		
			MVISIO::swap();		
			Sleep(20); // Don't kill mthreading ;)
	}      
   
   // Done:
   MVISIO::free();
   return 0;
}
